!! module to implemet the micro genetic algorithms option 

module ga_microga
	use ga_aux

	implicit none

	contains

	!-----------------------------------------------------------------------
	subroutine gamicro(nparam,npopsiz,nchrome,nowrite,i,npossum,ig2sum, &
		& ibest,diffrac,parent,iparent,g0,g1,ig2,nposibl,parmax)
	!-----------------------------------------------------------------------
	! Micro-GA implementation subroutine
	!-----------------------------------------------------------------------

		!! subroutine arguments
		integer, intent(in) :: nparam
		integer, intent(in) :: i
		integer, intent(in) :: npossum
		integer, intent(in) :: ig2sum
		integer, intent(in) :: npopsiz
		integer, intent(in) :: nchrome
		integer, intent(in) :: nowrite
		real, intent(in) 	:: diffrac
		integer,dimension(nchrome,npopsiz),intent(out)	:: iparent
		integer,dimension(nchrome),intent(in)			:: ibest
		integer,dimension(nparam),intent(in)			:: nposibl
        integer,dimension(nparam),intent(in)			:: ig2
		real,dimension(nparam),intent(in)				:: g0
        real,dimension(nparam),intent(in)				:: g1
		real,dimension(nparam,npopsiz),intent(in)		:: parent
		real,dimension(nparam),intent(in)				:: parmax

		!! local variables
		integer,save :: j,n,icount
		real,save		:: diffrac1
		real,save		:: rand	


		! first, check for convergence of micro population.
		! If converged, start a new generation with best individual and fill
		! the remainder of the population with new randomly generated parents.

		!count number of bits from each individual which are different from
		! the corresponding bits of the best member in the micro-population
		icount=0
		do j=1,npopsiz
			do n=1,nchrome
				if(iparent(n,j) /= ibest(n)) icount=icount+1
			end do
		end do

		! if icount less than diffrac % of number of bits, then consider population
		! to have converged.  Restart with best individual and random others.
		diffrac1=dble(icount)/dble((npopsiz-1)*nchrome)
		if (diffrac1 < diffrac) then

			! save best
			iparent(1:nchrome,1)=ibest(1:nchrome)		

			! randomly generate the others
			do j=2,npopsiz
				do n=1,nchrome
					call ran3(1,rand)
					iparent(n,j)=1
					if(rand < 0.5d0) iparent(n,j)=0
				end do
			end do

			! make sure that new population is within bounds. This is only
			! necessary if the number of possibilities per parameter (that is,
			! the range spanned by its number of bits) is not optimized to
			! be 2**n, i.e. npossum < ig2sum 
			if (npossum < ig2sum) then
				call possibl(nparam,nchrome,npopsiz,nowrite,&
					&	parent,iparent,g0,g1,ig2,nposibl,parmax)
			end if

			write(0,1375) i
			write(24,1375) i
		end if

		include 'formats'

		return
	end subroutine gamicro
	!-------------------------------------------------------------------------
end module ga_microga
